@@ -54,7 +54,7 @@ static bool shouldOverrideChar(QChar ch)
 
 static bool isCompleteStringLiteral(const BackwardsScanner &tk, int index)
 {
-    const QStringRef text = tk.textRef(index);
+    const QStringView text = tk.textRef(index);
 
     if (text.length() < 2)
         return false;
@@ -128,7 +128,7 @@ static int countSkippedChars(const QString &blockText, const QString &textToProc
     return skippedChars;
 }
 
-static const Token tokenAtPosition(const Tokens &tokens, const unsigned pos)
+static const Token tokenAtPosition(const Tokens &tokens, const int pos)
 {
     for (int i = tokens.size() - 1; i >= 0; --i) {
         const Token tk = tokens.at(i);
@@ -145,6 +145,7 @@ static LanguageFeatures languageFeatures()
     features.qtKeywordsEnabled = false;
     features.qtMocRunEnabled = false;
     features.cxx11Enabled = true;
+    features.cxx14Enabled = true;
     features.cxxEnabled = true;
     features.c99Enabled = true;
     features.objCEnabled = true;
@@ -449,7 +450,7 @@ bool MatchingText::contextAllowsElectricCharacters(const QTextCursor &cursor)
         return false;
 
     if (token.isStringLiteral() || token.isCharLiteral()) {
-        const unsigned pos = cursor.selectionEnd() - cursor.block().position();
+        const int pos = cursor.selectionEnd() - cursor.block().position();
         if (pos <= token.utf16charsEnd())
             return false;
     }
@@ -484,7 +485,7 @@ bool MatchingText::isInCommentHelper(const QTextCursor &cursor, Token *retToken)
     int prevState = 0;
     const Tokens tokens = getTokens(cursor, prevState);
 
-    const unsigned pos = cursor.selectionEnd() - cursor.block().position();
+    const int pos = cursor.selectionEnd() - cursor.block().position();
 
     if (tokens.isEmpty() || pos < tokens.first().utf16charsBegin())
         return prevState > 0;
@@ -509,7 +510,7 @@ Kind MatchingText::stringKindAtCursor(const QTextCursor &cursor)
     int prevState = 0;
     const Tokens tokens = getTokens(cursor, prevState);
 
-    const unsigned pos = cursor.selectionEnd() - cursor.block().position();
+    const int pos = cursor.selectionEnd() - cursor.block().position();
 
     if (tokens.isEmpty() || pos <= tokens.first().utf16charsBegin())
         return T_EOF_SYMBOL;
@@ -542,7 +543,7 @@ QString MatchingText::insertMatchingBrace(const QTextCursor &cursor, const QStri
     }
 
     QString result;
-    foreach (const QChar &ch, text) {
+    for (const QChar &ch : qAsConst(text)) {
         if      (ch == QLatin1Char('('))  result += QLatin1Char(')');
         else if (ch == QLatin1Char('['))  result += QLatin1Char(']');
         else if (ch == QLatin1Char('{'))  result += QLatin1Char('}');
