@@ -70,6 +70,7 @@
 #include <texteditor/texteditoractionhandler.h>
 #include <texteditor/textmark.h>
 
+#include <utils/algorithm.h>
 #include <utils/delegates.h>
 #include <utils/changeset.h>
 #include <utils/qtcassert.h>
@@ -130,8 +131,6 @@ void QmlJSEditorWidget::finalizeInitialization()
     connect(&m_updateOutlineIndexTimer, &QTimer::timeout,
             this, &QmlJSEditorWidget::updateOutlineIndexNow);
 
-    textDocument()->setCodec(QTextCodec::codecForName("UTF-8")); // qml files are defined to be utf-8
-
     m_modelManager = ModelManagerInterface::instance();
     m_contextPane = Internal::QmlJSEditorPlugin::quickToolBar();
 
@@ -190,7 +189,7 @@ static void appendExtraSelectionsForMessages(
         const QList<DiagnosticMessage> &messages,
         const QTextDocument *document)
 {
-    foreach (const DiagnosticMessage &d, messages) {
+    for (const DiagnosticMessage &d : messages) {
         const int line = d.loc.startLine;
         const int column = qMax(1U, d.loc.startColumn);
 
@@ -209,7 +208,7 @@ static void appendExtraSelectionsForMessages(
             sel.cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, d.loc.length);
         }
 
-        const auto fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+        const auto fontSettings = TextEditor::TextEditorSettings::fontSettings();
 
         if (d.isWarning())
             sel.format = fontSettings.toTextCharFormat(TextEditor::C_WARNING);
@@ -373,8 +372,13 @@ void QmlJSEditorWidget::updateUses()
         return;
 
     QList<QTextEdit::ExtraSelection> selections;
-    foreach (const SourceLocation &loc,
-             m_qmlJsEditorDocument->semanticInfo().idLocations.value(wordUnderCursor())) {
+    QList<SourceLocation> locations
+            = m_qmlJsEditorDocument->semanticInfo().idLocations.value(wordUnderCursor());
+    // code model may present the locations not in a document order
+    Utils::sort(locations, [](const SourceLocation &lhs, const SourceLocation &rhs) {
+        return lhs.begin() < rhs.begin();
+    });
+    for (const SourceLocation &loc : qAsConst(locations)) {
         if (! loc.isValid())
             continue;
 
@@ -411,7 +415,7 @@ protected:
     {
         UiQualifiedId *id = qualifiedTypeNameId(member);
         if (id) {
-            const QStringView &name = id->name;
+            QStringView name = id->name;
             if (!name.isEmpty() && name.at(0).isUpper())
                 return true;
         }
@@ -429,7 +433,7 @@ protected:
             else if (script->qualifiedId->next)
                 return false;
 
-            const QStringView &propertyName = script->qualifiedId->name;
+            QStringView propertyName = script->qualifiedId->name;
 
             if (propertyName == QLatin1String("id"))
                 return true;
@@ -507,10 +511,10 @@ void QmlJSEditorWidget::setSelectedElements()
 
     if (m_qmlJsEditorDocument->semanticInfo().isValid()) {
         SelectedElement selectedMembers;
-        QList<UiObjectMember *> members
-                = selectedMembers(m_qmlJsEditorDocument->semanticInfo().document, startPos, endPos);
+        const QList<UiObjectMember *> members
+            = selectedMembers(m_qmlJsEditorDocument->semanticInfo().document, startPos, endPos);
         if (!members.isEmpty()) {
-            foreach (UiObjectMember *m, members) {
+            for (UiObjectMember *m : members) {
                 offsets << m;
             }
         }
@@ -749,7 +753,7 @@ void QmlJSEditorWidget::inspectElementUnderCursor() const
 }
 
 void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
-                                   Utils::ProcessLinkCallback &&processLinkCallback,
+                                   const Utils::LinkHandler &processLinkCallback,
                                    bool /*resolveTarget*/,
                                    bool /*inNextSplit*/)
 {
@@ -764,9 +768,11 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
 
     if (auto importAst = cast<const AST::UiImport *>(node)) {
         // if it's a file import, link to the file
-        foreach (const ImportInfo &import, semanticInfo.document->bind()->imports()) {
+        const QList<ImportInfo> imports = semanticInfo.document->bind()->imports();
+        for (const ImportInfo &import : imports) {
             if (import.ast() == importAst && import.type() == ImportType::File) {
-                Utils::Link link(import.path());
+                Utils::Link link(
+                    m_modelManager->fileToSource(FilePath::fromString(import.path())));
                 link.linkTextStart = importAst->firstSourceLocation().begin();
                 link.linkTextEnd = importAst->lastSourceLocation().end();
                 processLinkCallback(Utils::Link());
@@ -784,15 +790,13 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
         link.linkTextStart = literal->literalToken.begin();
         link.linkTextEnd = literal->literalToken.end();
         if (semanticInfo.snapshot.document(text)) {
-            link.targetFileName = text;
+            link.targetFilePath = Utils::FilePath::fromString(text);
             processLinkCallback(link);
             return;
         }
-        const QString relative = QString::fromLatin1("%1/%2").arg(
-                    semanticInfo.document->path(),
-                    text);
-        if (QFileInfo::exists(relative)) {
-            link.targetFileName = relative;
+        const Utils::FilePath relative = Utils::FilePath::fromString(semanticInfo.document->path()).pathAppended(text);
+        if (relative.exists()) {
+            link.targetFilePath = m_modelManager->fileToSource(relative);
             processLinkCallback(link);
             return;
         }
@@ -809,7 +813,7 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
         return processLinkCallback(Utils::Link());
 
     Utils::Link link;
-    link.targetFileName = fileName;
+    link.targetFilePath = m_modelManager->fileToSource(FilePath::fromString(fileName));
     link.targetLine = line;
     link.targetColumn = column - 1; // adjust the column
 
@@ -874,7 +878,7 @@ void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
         AssistInterface *interface = createAssistInterface(QuickFix, ExplicitlyInvoked);
         if (interface) {
             QScopedPointer<IAssistProcessor> processor(
-                        Internal::QmlJSEditorPlugin::quickFixAssistProvider()->createProcessor());
+                Internal::QmlJSEditorPlugin::quickFixAssistProvider()->createProcessor(interface));
             QScopedPointer<IAssistProposal> proposal(processor->perform(interface));
             if (!proposal.isNull()) {
                 GenericProposalModelPtr model = proposal->model().staticCast<GenericProposalModel>();
@@ -892,7 +896,8 @@ void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 
     if (ActionContainer *mcontext = ActionManager::actionContainer(Constants::M_CONTEXT)) {
         QMenu *contextMenu = mcontext->menu();
-        foreach (QAction *action, contextMenu->actions()) {
+        const QList<QAction *> actions = contextMenu->actions();
+        for (QAction *action : actions) {
             menu->addAction(action);
             if (action->objectName() == QLatin1String(Constants::M_REFACTORING_MENU_INSERTION_POINT))
                 menu->addMenu(refactoringMenu);
@@ -1017,11 +1022,8 @@ AssistInterface *QmlJSEditorWidget::createAssistInterface(
     AssistReason reason) const
 {
     if (assistKind == Completion) {
-        return new QmlJSCompletionAssistInterface(document(),
-                                                  position(),
-                                                  textDocument()->filePath(),
-                                                  reason,
-                                                  m_qmlJsEditorDocument->semanticInfo());
+        return new QmlJSCompletionAssistInterface(textCursor(), textDocument()->filePath(),
+                                                  reason, m_qmlJsEditorDocument->semanticInfo());
     } else if (assistKind == QuickFix) {
         return new Internal::QmlJSQuickFixAssistInterface(const_cast<QmlJSEditorWidget *>(this), reason);
     }
